//
// Created by robin on 7/22/16.
//

#ifndef LIBNFC_CPPTOOLS_FREEFARETAGBUSINESS_H
#define LIBNFC_CPPTOOLS_FREEFARETAGBUSINESS_H


#include <boost/shared_ptr.hpp>
#include <libnfc_cpptools/FreeFareSector.h>

namespace LibNfc
{
namespace FreeFare
{

class FreeFareTagInternal;

typedef std::vector<std::pair<std::string, std::string>> MappedKeys;

class FreeFareTag
{
public:
    enum FreeFareTagType {
        UNKNOWN,
        FELICA,
        MIFARE_MINI,
        MIFARE_CLASSIC_1K,
        MIFARE_CLASSIC_4K,
        MIFARE_DESFIRE,
        MIFARE_ULTRALIGHT,
        MIFARE_ULTRALIGHT_C
    };

    static std::string getTagTypeString(FreeFareTagType type);

    explicit FreeFareTag(std::shared_ptr<FreeFareTagInternal> tag);
    ~FreeFareTag() = default;

    LibNfc::Utils::ResultBool authenticate(int sector, const std::string& key, int keyType);

    LibNfc::Utils::Result<MappedKeys> mapKeys(const std::vector<std::string>& keys, std::function<void(int, int)> cb = nullptr);

    LibNfc::Utils::ResultString readBlock(int sector, int block, const std::string& key, int keyType);

    LibNfc::Utils::Result<FreeFareSector> readSector(int sector, const std::string& key, int keyType);

    LibNfc::Utils::Result<std::vector<FreeFareSector>> read(const MappedKeys& keys, std::function<void(int, int)> cb = nullptr);

    LibNfc::Utils::Result<std::vector<FreeFareSector>> read(const std::vector<std::string>& keys, std::function<void(int, int)> mapCb = nullptr,
                                        std::function<void(int, int)> readCb = nullptr);

    LibNfc::Utils::ResultBool writeBlock(int sector, int block, const std::string& key, int keyType, const std::string& data);

    LibNfc::Utils::ResultBool writeSector(int sector, const std::string& key, int keyType, const std::string& data);

    LibNfc::Utils::ResultBool write(const MappedKeys& keys, const std::string& data, bool writeSector0, std::function<void(int, int)> cb = nullptr);

    LibNfc::Utils::ResultBool write(const std::vector<std::string>& keys, const std::string& data, bool writeSector0,
                     std::function<void(int, int)> mapCb = nullptr, std::function<void(int, int)> writeCb = nullptr);

    const std::string& getUid() const;

    FreeFareTagType getType() const;

    std::shared_ptr<FreeFareTagInternal> getTag() const;

protected:
    std::shared_ptr<FreeFareTagInternal> _tag;
};

}; // FreeFare
}; // LibNfc

#endif //LIBNFC_CPPTOOLS_FREEFARETAGBUSINESS_H
